/*  
 *  CDL.H -- Global definitions for the Client Display Library.
 */

#define	CDL_VERSION	"Client Display Library V1.0 4/27/97"

/*  Default values, size limiting values.
 */
#define	SZ_NAME		256		/* size of an image name	*/
#define	SZ_LINE		256		/* size of temp buffer		*/
#define	SZ_IMTDEV	128		/* size of an IMTDEV string 	*/
#define MAX_FBCONFIG	128		/* max size of FB config table 	*/
#define MAX_FRAMES	4		/* max frames support by server	*/
#define	OK		0		/* success			*/
#define	ERR		1		/* failure			*/
#define	EOS		'\0'		/* end of statement		*/
#define	DEF_CONTRAST	0.25		/* default zscale contrast	*/
#define	DEF_NSAMPLE	600		/* default number of samples	*/
#define	DEF_NSAMPLINES	  5		/* default no. of sample lines	*/

#define	FB_AUTO		-1		/* autoconfig the frame buffer	*/

/* Types of greyscale transformations. */
#define CDL_UNITARY       0            	/* values map without change	*/
#define CDL_LINEAR        1            	/* linear mapping		*/
#define CDL_LOG           2            	/* logarithmic mapping	*/

/* Overlay colors. */
#define	C_BLACK		202		/* static overlay color defs	*/
#define	C_WHITE		203
#define	C_RED		204
#define	C_GREEN		205
#define	C_BLUE		206
#define C_YELLOW	207
#define	C_CYAN		208
#define C_MAGENTA	209
#define C_CORAL		210
#define	C_MAROON	211
#define C_ORANGE	212
#define	C_KHAKI		213
#define C_ORCHID	214
#define C_TURQUOISE	215
#define C_VIOLET	216
#define	C_WHEAT		217

/* Overlay point mark types. */
#define M_FILL             1
#define M_POINT            2
#define M_BOX              4
#define M_PLUS             8
#define M_CROSS           16
#define M_DIAMOND         32
#define M_CIRCLE          64
#define M_STAR           128
#define M_HLINE          256
#define M_VLINE          512
#define M_HBLINE        1024
#define M_VBLINE        2048


/* Local type definitions. */
#ifndef AIXV3
typedef	unsigned char	uchar;
#endif
typedef	struct CDL 	*CDLPtr;
typedef	struct Marker 	*MarkerPtr;


/* Define the structures needed by the IMD interface.  This is a kludge
 * to avoid having applications include the IMD header.
 */

#define MAX_FBCONFIG    128        

typedef struct {
    int config;                         /* configuration number         */
    int nframes;                        /* number of frames             */
    int width, height;                  /* frame buffer dimensions      */
} FBTab;

struct IMD {
        int     datain, dataout;        /* connection file descriptors  */
        int     domain;                 /* connection type              */

        short   frame;                  /* display frame number         */
        short   fbconfig;               /* frame buffer config number   */
        FBTab   *fbtab[MAX_FBCONFIG];   /* fb configuration table       */

        short   xs, xe;                 /* X start/end values           */
        short   ys, ye;                 /* Y start/end values           */
        char    *name;                  /* image name                   */
        char    *title;                 /* image title                  */
        float   a, b, c, d;             /* WCS values                   */
        float   tx, ty;                 /* translation values           */
        float   z1, z2;                 /* zscale values                */
        short   ztrans;                 /* Z trans type                 */

};
typedef struct IMD *IMDPtr;


/*  The main CDL package structure.
 */
struct CDL {
	IMDPtr	imd;			/* IMD package pointer	*/
	int	memModel;		/* Memory model to use	*/

	int	frame;			/* display frame number	*/
	int	fbconfig;		/* fb config number 	*/
	int	fbwidth;		/* current FB width	*/
	int	fbheight;		/* current FB height	*/
	int	fbnf;			/* current FB nframes   */

	float	contrast;		/* zscale contrast value*/
	int	nsample;		/* opt. sample points	*/
	int	nsamplines;		/* opt. sample lines	*/

	int	im_nx;	   		/* current image width	*/
	int	im_ny;	   		/* current image height	*/

	/* WCS descriptor stuff. */
	char	*imname;		/* image name		*/
	char	*imtitle;		/* image title		*/
	float	a, b, c, d;		/* WCS values 		*/
	float	tx, ty;			/* translation values	*/
	float	z1, z2;			/* zscale values	*/
	int	ztrans;			/* Z trans type		*/
};


/* A element of the display list describing the marker.  We throw in the
 * kitchen sinke here to cover all possible marker types, in reality only
 * a few of these are used for any given marker type.
 */
struct Marker {
	short	type;			/* marker type		*/
	int	x, y;			/* center coords	*/
	int	number;			/* label a point	*/
	int	pt_type;		/* point type		*/
	int	size;			/* point marker size	*/
	int	fill;			/* fill marker 		*/
	int	color;			/* marker color		*/
	int	*xp, *yp;		/* coords		*/
	int	npts;			/* npts in array	*/
	int	radius;			/* circle radius	*/
	int	nannuli;		/* num of annuli	*/
	int	sep;			/* annulus separation	*/
	int	xrad, yrad;		/* ellipse axes		*/
	float	ang;			/* ellipse rotation	*/
	float	txsize;			/* text marker size	*/
	char	*str;			/* text marker string	*/

	int	nx, ny;			/* marker region size	*/
	int	lx, ly;			/* marker LL corner	*/

	uchar	*refpix;		/* orig image pixels	*/
	uchar	*markpix;		/* marked image pixels	*/

	MarkerPtr  back;		/* linked list pointers	*/
	MarkerPtr  next;
	int	markerID;		/* assigned id number 	*/
};


/* Marker types used internally. */
#define MK_POINT	0
#define MK_LINE		1
#define MK_BOX		2
#define MK_POLYLINE	3
#define MK_POLYGON	4
#define MK_CIRCLE	5
#define MK_CIRCANN	6
#define MK_ELLIPSE	7
#define MK_ELLIPANN	8
#define MK_TEXT		9

/* Utility macros.
 */
#undef	max
#define	max(a,b)	(a > b ? a : b)
#undef	min
#define	min(a,b)	(a < b ? a : b)

/* Handy values.
 */
#undef	True
#define	True		1
#undef  False
#define False		0


/* Function definitions. */
CDLPtr  cdl_open();

char	cdl_readCursor();

int	cdl_displayPix(), cdl_displayIRAF(), cdl_displayFITS();
int	cdl_readIRAF(), cdl_readFITS(), cdl_clearFrame();
int	cdl_readImage(), cdl_readFrameBuffer(), cdl_readSubRaster();
int	cdl_writeSubRaster(), cdl_setCursor();
int	cdl_printPix(), cdl_printPixToFile();
int	cdl_setWCS(), cdl_getWCS();

void	cdl_selectFB(), cdl_close(), cdl_computeZscale(), cdl_zscaleImage();
void	cdl_setFrame(), cdl_setFBConfig(), cdl_setZTrans(), cdl_setZoom();
void	cdl_setSample(), cdl_setContrast(), cdl_setName(), cdl_setTitle();
void	cdl_getFrame(), cdl_getFBConfig(), cdl_getZTrans(), cdl_getZoom();
void	cdl_getSample(), cdl_getContrast(), cdl_getName(), cdl_getTitle();
void	cdl_getSampleLines(), cdl_setSampleLines(), cdl_zscale();
void	cdl_setZScale(), cdl_getZScale(), cdl_lookupFBSize();
void	cdl_setDebug();

/* Marker function definitions. */
int	cdl_markPoint(), cdl_markLine(), cdl_markBox(), cdl_markPolyline();
int	cdl_markPolygon(), cdl_markCircle(), cdl_markCircAnnuli();
int	cdl_markEllipse(), cdl_markEllipAnnuli(), cdl_markText();
int	cdl_deleteMark(), cdl_clearOverlay(), cdl_redrawOverlay();


/* Define a grayscale+static colormap used by the print routines. */
static uchar cmap_r[] = {
  0,   1,   3,   4,   5,   6,   8,   9,  10,  12,  13,  14,  15,  17,  18,  19,
 20,  22,  23,  24,  26,  27,  28,  29,  31,  32,  33,  35,  36,  37,  38,  40,
 41,  42,  44,  45,  46,  47,  49,  50,  51,  52,  54,  55,  56,  58,  59,  60,
 61,  63,  64,  65,  67,  68,  69,  70,  72,  73,  74,  76,  77,  78,  79,  81,
 82,  83,  84,  86,  87,  88,  90,  91,  92,  93,  95,  96,  97,  99, 100, 101,
102, 104, 105, 106, 108, 109, 110, 111, 113, 114, 115, 116, 118, 119, 120, 122,
123, 124, 125, 127, 128, 129, 131, 132, 133, 134, 136, 137, 138, 140, 141, 142,
143, 145, 146, 147, 148, 150, 151, 152, 154, 155, 156, 157, 159, 160, 161, 163,
164, 165, 166, 168, 169, 170, 172, 173, 174, 175, 177, 178, 179, 180, 182, 183,
184, 186, 187, 188, 189, 191, 192, 193, 195, 196, 197, 198, 200, 201, 202, 204,
205, 206, 207, 209, 210, 211, 212, 214, 215, 216, 218, 219, 220, 221, 223, 224,
225, 227, 228, 229, 230, 232, 233, 234, 236, 237, 238, 239, 241, 242, 243, 244,
246, 247, 248, 250, 251, 252, 253, 255, 255, 255,   0, 255, 255,   0,   0, 255,
  0, 255, 255, 177, 255, 255, 219,   0, 239, 255,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0
}; 

static uchar cmap_g[] = {
  0,   1,   3,   4,   5,   6,   8,   9,  10,  12,  13,  14,  15,  17,  18,  19,
 20,  22,  23,  24,  26,  27,  28,  29,  31,  32,  33,  35,  36,  37,  38,  40,
 41,  42,  44,  45,  46,  47,  49,  50,  51,  52,  54,  55,  56,  58,  59,  60,
 61,  63,  64,  65,  67,  68,  69,  70,  72,  73,  74,  76,  77,  78,  79,  81,
 82,  83,  84,  86,  87,  88,  90,  91,  92,  93,  95,  96,  97,  99, 100, 101,
102, 104, 105, 106, 108, 109, 110, 111, 113, 114, 115, 116, 118, 119, 120, 122,
123, 124, 125, 127, 128, 129, 131, 132, 133, 134, 136, 137, 138, 140, 141, 142,
143, 145, 146, 147, 148, 150, 151, 152, 154, 155, 156, 157, 159, 160, 161, 163,
164, 165, 166, 168, 169, 170, 172, 173, 174, 175, 177, 178, 179, 180, 182, 183,
184, 186, 187, 188, 189, 191, 192, 193, 195, 196, 197, 198, 200, 201, 202, 204,
205, 206, 207, 209, 210, 211, 212, 214, 215, 216, 218, 219, 220, 221, 223, 224,
225, 227, 228, 229, 230, 232, 233, 234, 236, 237, 238, 239, 241, 242, 243, 244,
246, 247, 248, 250, 251, 252, 253, 255, 255, 255,   0, 255,   0, 255,   0, 255,
255,   0, 127,  48, 166, 247, 112, 246, 131, 232,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 
};

static uchar cmap_b[] = {
  0,   1,   3,   4,   5,   6,   8,   9,  10,  12,  13,  14,  15,  17,  18,  19,
 20,  22,  23,  24,  26,  27,  28,  29,  31,  32,  33,  35,  36,  37,  38,  40,
 41,  42,  44,  45,  46,  47,  49,  50,  51,  52,  54,  55,  56,  58,  59,  60,
 61,  63,  64,  65,  67,  68,  69,  70,  72,  73,  74,  76,  77,  78,  79,  81,
 82,  83,  84,  86,  87,  88,  90,  91,  92,  93,  95,  96,  97,  99, 100, 101,
102, 104, 105, 106, 108, 109, 110, 111, 113, 114, 115, 116, 118, 119, 120, 122,
123, 124, 125, 127, 128, 129, 131, 132, 133, 134, 136, 137, 138, 140, 141, 142,
143, 145, 146, 147, 148, 150, 151, 152, 154, 155, 156, 157, 159, 160, 161, 163,
164, 165, 166, 168, 169, 170, 172, 173, 174, 175, 177, 178, 179, 180, 182, 183,
184, 186, 187, 188, 189, 191, 192, 193, 195, 196, 197, 198, 200, 201, 202, 204,
205, 206, 207, 209, 210, 211, 212, 214, 215, 216, 218, 219, 220, 221, 223, 224,
225, 227, 228, 229, 230, 232, 233, 234, 236, 237, 238, 239, 241, 242, 243, 244,
246, 247, 248, 250, 251, 252, 253, 255, 255, 255,   0, 255,   0,   0, 255,   0,
255, 255,  80,  96,   0, 144, 215, 255, 239, 187,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0
};

